-----------------------------------
-- Henge - Generic Necrosis-like --
-----------------------------------

Henge = AceLibrary("AceAddon-2.0"):new(
	"AceConsole-2.0",
	"AceDB-2.0",
	"AceDebug-2.0",
	"AceEvent-2.0",
	"AceModuleCore-2.0"
)
local L  = AceLibrary("AceLocale-2.2"):new("Henge")

local AceOO = AceLibrary("AceOO-2.0")
local Dewdrop = AceLibrary("Dewdrop-2.0")
local Tablet = AceLibrary("Tablet-2.0")
local PT = AceLibrary("PeriodicTable-2.0")

local eventFuncs = {
	["LearnSpell"] = {},
	["BagSlotUpdate"] = {}
}

function Henge:OnInitialize()
	self:SetDebugging(true)
	self.classNames = {
		["WARRIOR"] = "RageHenge",
		["HUNTER"]  = "BeastHenge",
		["ROGUE"]   = "StealthHenge",
		["PRIEST"]  = "FaithHenge",
		["MAGE"]    = "RuneHenge",
		["DRUID"]   = "TreeHenge",
		["SHAMAN"]  = "TotemHenge",
		["PALADIN"] = "BubbleHenge",
		["WARLOCK"] = "StoneHenge"
	}
	self.info = {
		["name"] = "Henge",
		["icon"] = "Interface\\AddOns\\Henge\\icon"
	}
	self.spellInfo = {}
	self.itemInfo = {}
	_, self.myClass = UnitClass("player")
	if self.classNames[self.myClass] then
		self.info.name = self.classNames[self.myClass]
	end
	for loop = 1, GetNumAddOns(), 1 do
		local loadIt = true
		local xHengeModule = GetAddOnMetadata(loop, "X-Henge-Module")
		if not xHengeModule then
			loadIt = false
		else
			local xHengeClass = GetAddOnMetadata(loop, "X-Henge-Class")
			if xHengeClass then
				if strupper(xHengeClass) ~= self.myClass then
					loadIt = false
				end
			end
		end
		if loadIt == true and IsAddOnLoadOnDemand(loop) then
			LoadAddOn(loop)
		end
	end
	self.loadedModules = {}
	for mname, module in self:IterateModules() do
		if self:IsModuleActive(module) then
			if (not type(module.Test) == "function") or module:Test() then
				self.loadedModules[mname] = module
			end
		end
	end
	self:RegisterDB("HengeDB", "HengePCDB")
	self:RegisterEvent("SpecialEvents_LearnSpell")
	self:RegisterEvent("SpecialEvents_BagSlotUpdate")
	self:RegisterEvent("PLAYER_ALIVE", "UpdateAll")
end

function Henge:OnEnable()
	HengeSphere:AddData({
		["id"]   = "none",
		["name"] = L["None"],
		["func"] = function() return 0, 0, false end,
		["event"] = nil
	})
	HengeSphere:AddData({
		["id"]   = "health",
		["name"] = L["Health"],
		["func"] = function()
			return UnitHealth("player"), UnitHealthMax("player"), false
		end,
		["event"] = "UNIT_HEALTH",
		["arg1"] = "player"
	})
	HengeAction:Add({
		["id"]   = "none",
		["type"] = 'none',
		["name"] = L["None"]
	})
	self.defaults = {
		["profile"] = {
			["locked"] = false
		},
		["char"] = {
			["actopt"] = {}
		}
	}
	self:RegisterDefaults("profile", self.defaults.profile)
	self:RegisterDefaults("char", self.defaults.char)
	if self.db.profile.config_version ~= 3 then
		self:ResetDB("profile")
		self.db.profile.config_version = 3
	end
	if self.db.char.config_version ~= 3 then
		self:ResetDB("char")
		self.db.char.config_version = 3
	end
	self.optTable = {
		["type"] = 'group',
		["name"] = self.info.name,
		["icon"] = self.info.icon,
		["args"] = {
			["lock"] = {
				["type"] = 'toggle',
				["name"] = L["Lock Frames"],
				["desc"] = L["Prevent moving of the spheres and buttons."],
				["get"]  = function() return Henge:IsLocked() end,
				["set"]  = function() Henge:ToggleLocked() end,
				["order"] = 6
			},
			["update"] = {
				["type"] = 'execute',
				["name"] = L["Update Frames"],
				["desc"] = L["Force Henge to update its frames."],
				["func"] = function() Henge:UpdateAll() end,
				["order"] = 9
			},
			["reset"] = {
				["type"] = 'execute',
				["name"] = L["Reset Defaults"],
				["desc"] = L["Restore the default settings."],
				["func"] = function() Henge:ResetDefaults() end,
				["order"] = -2
			}
		}
	}
	self:RegisterChatCommand({"/henge"}, self.optTable)
	HengeSphere:Add({["id"] = "Main", ["name"] = "Main"})
	for mname, module in pairs(self.loadedModules) do
		module:Load()
		if mname == self.info.name then
			local path = "Interface\\AddOns\\"
			if IsAddOnLoaded(mname) then
				path = path..mname.."\\"
			else
				path = path.."Henge\\Modules\\"..mname.."\\"
			end
			self.info.icon = path..module.icon
			module.path = path
		end
	end
end

function Henge:GetThemeBase()
	return "Interface\\AddOns\\Henge\\images\\"
end

function Henge:ShowConfig()
end

function Henge:ResetDefaults()
	self:ResetDB("profile")
	self:ResetDB("char")
	ReloadUI()
end

function Henge:IsLocked()
	return self.db.profile.locked ~= false
end

function Henge:ToggleLocked()
	if self.db.profile.locked ~= false then
		self.db.profile.locked = false
	else
		self.db.profile.locked = true
	end
end

function Henge:UpdateAll()
	HengeSphere:UpdateAll()
	HengeButton:UpdateAll()
end

function Henge:SpecialEvents_LearnSpell()
	for k, v in pairs(eventFuncs.LearnSpell) do
		v()
	end
end

function Henge:SpecialEvents_BagSlotUpdate(bag, slot, itemlink, stack)
	for k, v in pairs(eventFuncs.BagSlotUpdate) do
		v()
	end
end

function Henge:LearnSpellFunc(id, func)
	eventFuncs.LearnSpell[id] = func
end

function Henge:BagSlotUpdateFunc(id, func)
	eventFuncs.BagSlotUpdate[id] = func
end
