----------------
-- RuneHenge --
----------------

local RuneHenge = Henge:NewModule("RuneHenge")
local ML, SL, PT = nil, nil, nil

function RuneHenge:Test()
	local _, pClass = UnitClass("player")
	return pClass == "MAGE"
end

function RuneHenge:Load()
	ML = HengeLocale:LoadRuneHenge()
	SL = AceLibrary("Babble-Spell-2.2")
	PT = AceLibrary("PeriodicTable-2.0")
	self.db = Henge:AcquireDBNamespace("RuneHenge")

	-- Set up defaults
	self.defaultsPC = {}
	self.defaults = {}
	Henge:RegisterDefaults("RuneHenge", "profile", self.defaults)
	Henge:RegisterDefaults("RuneHenge", "char", self.defaultsPC)

	self.fullname = "RuneHenge"
	self.icon     = "icon"

	local spell = {
	-- Buffs
		["arcaneint"]  =  SL["Arcane Intellect"],
		["dampen"]     =      SL["Dampen Magic"],
		["amplify"]    =     SL["Amplify Magic"],
		["manashield"] =       SL["Mana Shield"],
		["magearmor"]  =        SL["Mage Armor"],
		["brilliance"] = SL["Arcane Brilliance"],
		["fireward"]   =         SL["Fire Ward"],
		["frostarmor"] =       SL["Frost Armor"],
		["frostward"]  =        SL["Frost Ward"],
		["icearmor"]   =         SL["Ice Armor"],
		["icebarrier"] =       SL["Ice Barrier"],
	-- Conjuring
		["cwater"]    =        SL["Conjure Water"],
		["cfood"]     =         SL["Conjure Food"],
		["cmanagem1"] =   SL["Conjure Mana Agate"],
		["cmanagem2"] =    SL["Conjure Mana Jade"],
		["cmanagem3"] = SL["Conjure Mana Citrine"],
		["cmanagem4"] =    SL["Conjure Mana Ruby"],
	-- Teleport
		["teleif"]  =     SL["Teleport: Ironforge"],
		["teleorg"] =     SL["Teleport: Orgrimmar"],
		["telesw"]  =     SL["Teleport: Stormwind"],
		["teleuc"]  =     SL["Teleport: Undercity"],
		["teledar"] =     SL["Teleport: Darnassus"],
		["teletb"]  = SL["Teleport: Thunder Bluff"],
	-- Portals
		["portif"]  =     SL["Portal: Ironforge"],
		["portorg"] =     SL["Portal: Orgrimmar"],
		["portsw"]  =     SL["Portal: Stormwind"],
		["portuc"]  =     SL["Portal: Undercity"],
		["portdar"] =     SL["Portal: Darnassus"],
		["porttb"]  = SL["Portal: Thunder Bluff"],
	-- Other
		["polymorph"]   =           SL["Polymorph"],
		["slowfall"]    =           SL["Slow Fall"],
		["detect"]      =        SL["Detect Magic"],
		["rmvlsrcurse"] = SL["Remove Lesser Curse"],
		["blink"]       =               SL["Blink"],
		["polypig"]     =      SL["Polymorph: Pig"],
		["polyturtle"]  =   SL["Polymorph: Turtle"],
		["flamestrike"] =         SL["Flamestrike"],
		["blastwave"]   =          SL["Blast Wave"],
		["frostnova"]   =          SL["Frost Nova"],
		["blizzard"]    =            SL["Blizzard"]
	}
	for k, v in pairs(spell) do
		HengeAction:Add({
			["id"] = k,
			["name"] = v,
			["type"] = "spell",
			["spell"] = v
		})
	end
	HengeSphere:AddData("mana", {
		["func"] = function()
			return UnitMana("player"), UnitManaMax("player")
		end,
		["event"] = "UNIT_MANA",
		["arg1"] = "player"
	})
	local pos = {
		["conjgem"]   =  35,
		["fimarmor"]  = 140,
		["wards"]     = 175,
		["iceshibar"] = 210,
		["ampdampmg"] = 245
	}
	local data = {
		[1] = {
			["id"]   = "teleport",
			["type"] = 'menu',
			["name"] = ML["Teleport"],
			["desc"] = ML["Change destination for teleport."],
			["icon"] = self.icon,
			["pos"]  = 380
		},
		[2] = {
			["id"]   = "portal",
			["type"] = 'menu',
			["name"] = ML["Create Portal"],
			["desc"] = ML["Change destination for portal."],
			["icon"] = self.icon,
			["pos"]  = 315
		},
		[3] = {
			["id"]   = "arcintbri",
			["type"] = "action",
			["name"] = ML["Int. Buffs"],
			["desc"] = nil,
			["icon"] = "action:LeftButton",
			["pos"]  = 105,
			["acts"] = {
				["LeftButton"] = "arcaneint",
				["Shift-LeftButton"] = "brilliance"
			}
		},
		[4] = {
			["id"]   = "foodwater",
			["type"] = "action",
			["name"] = ML["Food/Water"],
			["desc"] = nil,
			["icon"] = "action:Shift-LeftButton",
			["pos"]  = 70,
			["acts"] = {
--				["LeftButton"] = "drinkcwater",
--				["RightButton"] = "eatcfood",
				["Shift-LeftButton"] = "cwater",
				["Shift-RightButton"] = "cfood"
			}
		}
	}
	if UnitFactionGroup("player") == "Alliance" then
		data[1].menu = {
			[1] = "teleif",
			[2] = "telesw",
			[3] = "teledar"
		}
		data[2].menu = {
			[1] = "portif",
			[2] = "portsw",
			[3] = "portdar"
		}
	else
		data[1].menu = {
			[1] = "teleorg",
			[2] = "teleuc",
			[3] = "teletb"
		}
		data[2].menu = {
			[1] = "portorg",
			[2] = "portuc",
			[3] = "porttb"
		}
	end
	for k, v in ipairs(data) do
		local bdata = {
			["id"]   = v.id,
			["type"] = v.type,
			["name"] = v.name,
			["desc"] = v.desc,
			["icon"] = v.icon,
			["options"] = {
				["anchor"] = "sphere:Main",
				["xpos"] = 0,
				["ypos"] = 0,
				["angle"] = v.pos
			}
		}
		if v.type == "menu" and type(v.menu) == "table" then
			bdata.children = {}
			for num, id in ipairs(v.menu) do
				bdata.children[num] = {
					["id"] = id,
					["name"] = spell[id],
					["actions"] = {
						["LeftButton"] = id
					},
					["options"] = {
					}
				}
			end
		elseif v.type == "action" and type(v.acts) == "table" then
			bdata.action = {}
			for mbs, act in pairs(v.acts) do
				bdata.action[mbs] = act
			end
		end
		HengeButton:Add(bdata)
	end
end
